var sugarRecipe = {};
Recipes.registryCandyRecipe = function(id, data, resultID, resultData) {
    sugarRecipe[id + ":" + data] = resultID + ":" + resultData;
};
Recipes.getCandyRecipeResult = function(id, data) {
    var _split = sugarRecipe[id + ":" + data];
    if (_split) {
        var id_data = _split.split(":");
        return {
            id: Number(id_data[0]),
            data: Number(id_data[1])
        };
    };
};

var burnerRecipe = {};
Recipes.registryBurnerRecipe = function(id, data, resultID, resultData) {
    burnerRecipe[id + ":" + data] = resultID + ":" + resultData;
};
Recipes.getBurnerRecipeResult = function(id, data) {
    var _split = burnerRecipe[id + ":" + data];
    if (_split) {
        var id_data = _split.split(":");
        return {
            id: Number(id_data[0]),
            data: Number(id_data[1])
        };
    };
};

var SugarCore = {
    createItem: function(id, itemName, dcr1, dcr2) {
        IDRegistry.genItemID(id);
        Item.createItem(id, itemName, dcr1, dcr2);
    },
    _ElectricNaturalRecipes: {},
    registerElectricNaturalRecipes: function(id, data, energy) {
        this._ElectricNaturalRecipes[id + ":" + data] = energy;
    },
    getElectricNaturalRecipes: function(id, data) {
        if (this._ElectricNaturalRecipes[id + ":" + data]) {
            return Number(this._ElectricNaturalRecipes[id + ":" + data]);
        };
    },
    createFoodItem: function(id, itemName, dcr1, dcr2) {
        IDRegistry.genItemID(id);
        Item.createFoodItem(id, itemName, dcr1, dcr2);
    },
    registerPrototype: function(id, Prototype) {
        ToolAPI.registerBlockMaterial(id, "stone", 1, true);
        Block.setDestroyTime(id, 3);
        TileEntity.registerPrototype(id, Prototype);
    },
    basicOutFunc: function(type, src) {
        this.data.last_energy_receive = this.data.energy_receive;
        this.data.energy_receive = 0;
        this.data.last_voltage = this.data.voltage;
        this.data.voltage = 0;
        var output = this.getEnergyLevel() * 15;
        if (this.data.energy >= output) {
            this.data.energy += src.add(output) - output;
        };
    },
    basicReceiveFunc: function(type, amount, voltage) {
        var add = Math.min(amount, this.getEnergyStorage() - this.data.energy);
        this.data.energy += add;
        this.data.energy_receive += add;
        this.data.voltage = Math.max(this.data.voltage, voltage);
        return add;
    },
    registerSUMachine: function(id, Prototype) {
        ICRender.getGroup("Sugar").add(id, -1);
        Prototype.defaultValues.energy = 0;
        Prototype.defaultValues.energy_receive = 0;
        Prototype.defaultValues.last_energy_receive = 0;
        Prototype.defaultValues.voltage = 0;
        Prototype.defaultValues.last_voltage = 0;
        Prototype.getEnergyLevel = Prototype.getEnergyLevel || function() {
            return 1;
        };;
        if (!Prototype.energyTick) {
            Prototype.energyTick = function() {
                this.data.last_energy_receive = this.data.energy_receive;
                this.data.energy_receive = 0;
                this.data.last_voltage = this.data.voltage;
                this.data.voltage = 0;
            };
        }
        Prototype.energyReceive = Prototype.energyReceive || this.basicReceiveFunc;
        this.registerPrototype(id, Prototype);
        EnergyTileRegistry.addEnergyTypeForId(id, SU);
    },
    registerSUStorage: function(id, Prototype) {
        Prototype.isEnergySource = function() {
                return true;
            };
        Prototype.energyReceive = Prototype.energyReceive || this.basicReceiveFunc;
        Prototype.energyTick = Prototype.energyTick || this.basicOutFunc;
        this.registerSUMachine(id, Prototype);
    },
    updateHeader: function(gui, text){
		var header = gui.getWindow("header");
		header.contentProvider.drawing[2].text = Translation.translate(text);
	}
};

/*
industrialCraft2
*/
var UpgradeAPI = {
    data: {},
    getUpgradeData: function(id) {
        return this.data[id];
    },
    isUpgrade: function(id) {
        return UpgradeAPI.data[id] ? true : false;
    },
    isValidUpgrade: function(id, count, data, container) {
        var upgrades = container.tileEntity.upgrades;
        var upgradeData = UpgradeAPI.getUpgradeData(id);
        if (upgradeData && (!upgrades || upgrades.indexOf(upgradeData.type) != -1)) {
            return true;
        }
        return false;
    },
    registerUpgrade: function(id, type, func) {
        this.data[id] = {
            type: type,
            func: func
        };
    },
    callUpgrade: function(item, machine, container, data) {
        var upgrades = machine.upgrades;
        var upgrade = this.getUpgradeData(item.id);
        if (upgrade && (!upgrades || upgrades.indexOf(upgrade.type) != -1)) {
            upgrade.func(item, machine, container, data);
        }
    },
    getUpgrades: function(machine, container) {
        var upgrades = [];
        for (var slotName in container.slots) {
            if (slotName.match(/Upgrade/)) {
                var slot = container.getSlot(slotName);
                if (slot.id > 0) {
                    var find = false;
                    for (var i in upgrades) {
                        var item = upgrades[i];
                        if (item.id == slot.id && item.data == slot.data) {
                            item.count += slot.count;
                            find = true;
                            break;
                        }
                    }
                    if (!find) {
                        item = {
                            id: slot.id,
                            count: slot.count,
                            data: slot.data
                        };
                        upgrades.push(item);
                    }
                }
            }
        }
        return upgrades;
    },
    executeUpgrades: function(machine) {
        var container = machine.container;
        var data = machine.data;
        var upgrades = this.getUpgrades(machine, container);
        for (var i in upgrades) {
            this.callUpgrade(upgrades[i], machine, container, data);
        }
        StorageInterface.checkHoppers(machine);
    },
}